"use strict";

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

/**
 * @module xbytes
 * @license Apache-2.0
 * @author Miraculous Owonubi
 * @copyright (c) 2019 Miraculous Owonubi
 */

/* eslint-disable no-restricted-properties */
function xbytes() {
  var size = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
  var options = arguments.length > 1 ? arguments[1] : undefined;
  if (Number.isNaN(size = +size) || typeof size !== 'number') return null;

  var _ref = options || {},
      _ref$iec = _ref.iec,
      iec = _ref$iec === void 0 ? false : _ref$iec,
      _ref$bits = _ref.bits,
      bits = _ref$bits === void 0 ? false : _ref$bits,
      _ref$short = _ref["short"],
      _short = _ref$short === void 0 ? true : _ref$short,
      _ref$space = _ref.space,
      space = _ref$space === void 0 ? true : _ref$space,
      _ref$fixed = _ref.fixed,
      fixed = _ref$fixed === void 0 ? 2 : _ref$fixed,
      prefixIndex = _ref.prefixIndex;

  size *= bits ? 8 : 1;
  var sizes = ['-', 'Kilo', 'Mega', 'Giga', 'Tera', 'Peta', 'Exa', 'Zetta', 'Yotta'];
  var exponent = typeof prefixIndex === 'number' ? prefixIndex : Math.floor(Math.log(Math.abs(size)) / Math.log(iec ? 1024 : 1000)) || 0;
  size /= ~exponent && exponent in sizes ? Math.pow(iec ? 1024 : 1000, exponent) : 1;
  var select = sizes[exponent] || sizes[0];
  var byteStr = !_short ? select.replace(/(\w{2})(\w+)/, "$1".concat(iec ? 'bi' : '$2', "-")).replace(/-/, !bits ? 'Bytes' : 'Bits') : select[0].replace(/(-?|[^\w])$/, "".concat(iec && exponent in sizes && exponent > 0 ? 'i' : '').concat(!bits ? 'B' : 'b'));
  return [size < 1 ? size : size.toFixed(fixed), byteStr].join(space ? ' ' : '');
}

var unitMatcher = /^(?:([kmgtpezy])(i?))?(b)/i;
var genericMatcher = new RegExp("\\b(\\d+(?:\\.\\d+)?)\\s*((?:([kmgtpezy])(i?))?(b))\\b");
var globalByteFilter = new RegExp(genericMatcher.source, 'gi');
var byteFilter = new RegExp("^".concat(genericMatcher.source), 'i');

function parseUnit(stringUnit) {
  var scan;

  if (typeof stringUnit === 'string' && (scan = stringUnit.match(unitMatcher))) {
    var _scan = scan,
        _scan2 = _slicedToArray(_scan, 4),
        input = _scan2[0],
        key = _scan2[1],
        iec = _scan2[2],
        type = _scan2[3];

    var prefix = key ? key.toUpperCase() : '' || type;
    scan = {
      input: input,
      prefix: prefix,
      iec: !!iec,
      type: type,
      bits: type === 'b',
      "byte": type === 'B',
      prefixIndex: 'BKMGTPEZY'.indexOf(prefix.toUpperCase())
    };
  }

  return scan;
}

function parseString(stringBytes) {
  var scan;

  if (typeof stringBytes === 'string' && (scan = stringBytes.match(byteFilter))) {
    var _scan3 = scan,
        _scan4 = _slicedToArray(_scan3, 3),
        input = _scan4[0],
        value = _scan4[1],
        unit = _scan4[2];

    scan = _objectSpread({}, parseUnit(unit), {
      input: input,
      value: parseFloat(value),
      unit: unit
    });
  }

  return scan;
}

function isBytes(stringBytes) {
  return !!parseString(stringBytes);
}

function extractBytes(stringOfbytes) {
  return (stringOfbytes.match(globalByteFilter) || []).filter(isBytes);
}

function parseSize(stringBytes, parseOptions) {
  var scan;

  var _ref2 = parseOptions || {},
      _ref2$bits = _ref2.bits,
      bits = _ref2$bits === void 0 ? true : _ref2$bits,
      _ref2$iec = _ref2.iec,
      iec = _ref2$iec === void 0 ? true : _ref2$iec;

  if (isBytes(stringBytes) && (scan = parseString(stringBytes))) {
    var index = scan.prefixIndex;
    scan = scan.value * (iec && !scan.iec ? Math.pow(10, index * 3) : Math.pow(2, 10 * index)) / (bits && scan.type === 'b' ? 8 : 1);
  }

  return scan;
}

function parseBytes(input, options) {
  var struct;
  var bytes = (isBytes(input) ? parseSize(input) : +input) || 0;

  if (isBytes(input)) {
    var _ref3 = [parseSize(input), input];
    bytes = _ref3[0];
    struct = _ref3[1];
  } else {
    var _ref4 = [+input || 0, xbytes(+input || 0)];
    bytes = _ref4[0];
    struct = _ref4[1];
  }

  return _objectSpread({}, struct = parseString(struct, options), {
    input: input,
    bytes: bytes,
    size: xbytes(bytes, struct)
  });
}

function relative(data, options) {
  var parsed = parseBytes(data).bytes;
  var bits = xbytes(parsed, _objectSpread({}, options, {
    iec: false,
    bits: true
  }));
  var bytes = xbytes(parsed, _objectSpread({}, options, {
    iec: false,
    bits: false
  }));
  var iecBits = xbytes(parsed, _objectSpread({}, options, {
    iec: true,
    bits: true
  }));
  var iecBytes = xbytes(parsed, _objectSpread({}, options, {
    iec: true,
    bits: false
  }));
  return {
    raw: data,
    size: parsed,
    bytes: bytes,
    iecBytes: iecBytes,
    bits: bits,
    iecBits: iecBits
  };
}

relative.bits = function (data) {
  return relative(data).bits;
};

relative.bytes = function (data) {
  return relative(data).bytes;
};

relative.iecBits = function (data) {
  return relative(data).iecBits;
};

relative.iecBytes = function (data) {
  return relative(data).iecBytes;
};

relative.size = function (size, unit, options) {
  var parsed = parseBytes(size);

  var _ref5 = parseUnit(unit) || {},
      prefixIndex = _ref5.prefixIndex,
      bits = _ref5.bits,
      iec = _ref5.iec;

  return xbytes(parsed.bytes, _objectSpread({}, options, {
    prefixIndex: prefixIndex,
    bits: bits,
    iec: iec
  }));
};

function createByteParser(config) {
  return function staticByteParser(size) {
    return xbytes(size, config);
  };
}

function createSizeParser(config) {
  return function staticSizeParser(size) {
    return parseSize(size, config);
  };
}

function createRelativeSizer(unit, config) {
  return function staticRelativeSizer(size) {
    return relative.size(size, unit, config);
  };
}

xbytes.byteFilter = byteFilter;
xbytes.unitMatcher = unitMatcher;
xbytes.genericMatcher = genericMatcher;
xbytes.globalByteFilter = globalByteFilter;
xbytes.isBytes = isBytes;
xbytes.relative = relative;
xbytes.extractBytes = extractBytes;
xbytes.parseSize = parseSize;
xbytes.parseUnit = parseUnit;
xbytes.parseBytes = parseBytes;
xbytes.parseString = parseString;
xbytes.createByteParser = createByteParser;
xbytes.createSizeParser = createSizeParser;
xbytes.createRelativeSizer = createRelativeSizer;
if (typeof module !== 'undefined') module.exports = xbytes;